import { EnforceOptions } from './definitions/enforce-options';
import { UniqueComparable } from './definitions/unique-comparable';
/**
 * Class that can be used to enforce unique values.
 *
 * @example
 *
 * const uniqueEnforcer = new UniqueEnforcer();
 *
 * const firstEmail = uniqueEnforcer.enforce(faker.internet.email())
 *
 * // You can be sure that secondEmail will always be different than firstEmail
 * const secondEmail = uniqueEnforcer.enforce(faker.internet.email())
 */
export declare class UniqueEnforcer {
    private store;
    /**
     * Function that can be used to enforce unique values.
     *
     * It accepts a value and returns it back if same value is not used before.
     * If the same value used before, it throws an {@link EnforceUniqueError}.
     *
     * @param value The value to enforce uniqueness.
     * @returns The same `value`
     * @throws `EnforceUniqueError` If the value used before.
     *
     * @example
     *
     * const uniqueEnforcer = new UniqueEnforcer();
     *
     * const firstEmail = uniqueEnforcer.enforce(faker.internet.email())
     *
     * // You can be sure that secondEmail will always be different than firstEmail.
     * // If faker.internet.email() returns the same value, `EnforceUniqueError` will be thrown.
     * const secondEmail = uniqueEnforcer.enforce(faker.internet.email())
     */
    enforce<T extends UniqueComparable>(value: T): T;
    /**
     * Function that can be used to enforce unique values.
     *
     * It accepts a value generator function, runs it to generate the value and returns it back if the same value is not used before or not excluded.
     * If the same value used before or excluded, it tries again until limits are exceeded.
     * It throws an {@link EnforceLimitError} when limits are exceeded.
     *
     * Limits can be configured with options parameter.
     *
     * @param generator Function to generate a value
     * @param options {@link EnforceOptions}
     * @returns The generated value
     * @throws `EnforceLimitError` If limits are exceeded and no unique value generated from the generator.
     *
     * @example
     *
     * const uniqueEnforcer = new UniqueEnforcer();
     *
     * function generateRandomNumber(): number {
     *   return Math.floor(Math.random() * 5) + 1;
     * }
     *
     * // You can be sure that num1 and num2 will be unique
     * const num1 = uniqueEnforcer.enforce(generateRandomNumber);
     * const num2 = uniqueEnforcer.enforce(generateRandomNumber);
     */
    enforce<T extends UniqueComparable>(generator: () => T, options?: EnforceOptions): T;
    /**
     * Resets the internal store of unique values in the UniqueEnforcer instance.
     * This method allows you to start enforcing uniqueness from scratch.
     *
     * @example
     *
     * const uniqueEnforcer = new UniqueEnforcer();
     *
     * const value1 = uniqueEnforcer.enforce('value');
     *
     * uniqueEnforcer.reset();
     *
     * // There will be no error thrown because of reset.
     * const value2 = uniqueEnforcer.enforce('value');
     */
    reset(): void;
}
